uniform float time;
uniform vec2 resolution;
uniform float boltsCount;
uniform float boltSpeed;
uniform float intensity;

float rand(vec2 co) {
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

float noise(vec2 p) {
    vec2 i = floor(p);
    vec2 f = fract(p);
    f = f * f * (3.0 - 2.0 * f);
    float a = rand(i);
    float b = rand(i + vec2(1.0, 0.0));
    float c = rand(i + vec2(0.0, 1.0));
    float d = rand(i + vec2(1.0, 1.0));
    return mix(mix(a, b, f.x), mix(c, d, f.x), f.y);
}

float fbm(vec2 p) {
    float v = 0.0;
    float a = 0.5;
    vec2 shift = vec2(100.0);
    mat2 rot = mat2(cos(0.5), sin(0.5), -sin(0.5), cos(0.5));
    for (int i = 0; i < 5; ++i) {
        v += a * noise(p);
        p = rot * p * 2.0 + shift;
        a *= 0.5;
    }
    return v;
}

float lightning(vec2 uv, float seed) {
    float bolt = 0.0;
    float t = time * boltSpeed + seed * 1000.0;
    vec2 p = uv;

    for(float i = 0.0; i < 3.0; i++) {
        float angle = fbm(vec2(t + i, seed)) * 6.28;
        vec2 dir = vec2(sin(angle), cos(angle));
        float dist = abs(dot(p, dir)) * (1.0 + fbm(p + t));
        bolt += 0.005 / (dist * dist);
        p += dir * 0.2;
    }

    float flicker = 0.5 + 0.5 * sin(t * 20.0);
    return bolt * flicker;
}

void main() {
    vec2 uv = (gl_FragCoord.xy * 2.0 - resolution.xy) / min(resolution.x, resolution.y);
    vec3 color = vec3(0.0);

    for(float i = 0.0; i < boltsCount; i++) {
        float t = time + i;
        vec2 offset = vec2(
            fbm(vec2(t, i)) * 2.0 - 1.0,
            fbm(vec2(i, t)) * 2.0 - 1.0
        );

        float bolt = lightning(uv + offset, i);
        vec3 boltColor = vec3(0.5, 0.7, 1.0) + 0.2 * sin(vec3(0.0, 0.3, 0.6) + i);
        color += bolt * boltColor * intensity;
    }

    gl_FragColor = vec4(color, 1.0);
}
